Object
 subclass: #PG8Demo
 instanceVariableNames: ' watch cancelled '
 category: 'PG8Demo'!
PG8Demo comment: '
	Implementation of demo for PhoneGap features.
	@2011 Alejandro Reimondo - aleReimondo@smalltalking.net
'!
PG8Demo class instanceVariableNames: 'current'!

! PG8Demo class methodsFor: 'accessing' !
current
	" Return the current instance of the receiver. "

	current isNil ifTrue: [ current := self new ].
	^current!

! PG8Demo class methodsFor: 'startup' !
startup
	" Perform startup actions. "

	(Smalltalk includesKey: #S8Web) ifTrue: [ self configureTools ].
	self current startup.! !

! PG8Demo class methodsFor: 'startup' !
configureTools
	" Private - Configure development tools. "

	S8Web addExpressions: #(
		#( '[PG8]--DEMO expressions------' '' )
		#( 'PhoneGap information' 'PG8Demo current showInformation.' )
		#( 'FileSystem Demo' 'PG8Demo current showFileSystemDemo.' )
		#( 'DataBase Demo' 'PG8Demo current showDataBaseDemo.' )
		#( 'File read/write Demo' 'PG8Demo current showFileDemo.' )
		#( 'Notification demo' 'PG8Demo current showNotificationDemo.' )
		#( 'Show acceleration' 'PG8Demo current showAcceleration.' )
		#( 'Watch acceleration' 'PG8Demo current watchAcceleration.' )
		#( 'Show compass' 'PG8Demo current showCompass.' )
		#( 'Watch compass' 'PG8Demo current watchCompass.' )
		#( 'Show geolocation' 'PG8Demo current showGeolocation.' )
		#( 'Watch geolocation' 'PG8Demo current watchGeolocation.' )
		#( 'Camera Demo' 'PG8Demo current showCameraDemo.' )
		#( 'Capture Demo' 'PG8Demo current showCaptureDemo.' )
	).
	S8Web initialCodeContents: '
	"Type your code here e.g.: Object selectors sorted  "
	#( #name #phonegap #platform #uuid #version ) collect: [:each| device[each] ].
	'.! !

! PG8Demo methodsFor: 'startup' !
startup
	" Private - Perform startup actions. "

	PhoneGap when: #deviceReady do: [ self showInformation ]! !

! PG8Demo methodsFor: 'GUI' !
outputView
	" Private - Return the HTML element for output. "

	^DOM document getElementById: #phoneGapResults! !

! PG8Demo methodsFor: 'cancel' !
cancelled
	" Return true if the receiver has been cancelled. "

	^cancelled = true! !

! PG8Demo methodsFor: 'cancel' !
cancel
	" Cancel the receiver. "

	cancelled := true! !

! PG8Demo methodsFor: 'io' !
clear
	" Clear output view. "

	cancelled := false.
	self outputView innerHTML: ' '! !

! PG8Demo methodsFor: 'io' !
cr
	" Emmit a new line in output view. "

	self nextPutAll: '<br />'! !

! PG8Demo methodsFor: 'io' !
hr
	" Emmit a new line in output view. "

	self nextPutAll: '<hr>'! !

! PG8Demo methodsFor: 'io' !
nextPutAll: aString
	" Emmit aString on output view. "

	self outputView innerHTML: self outputView innerHTML , aString! !

! PG8Demo methodsFor: 'io' !
title: aString level: aNumber
	" Emmit aString as tile on output view. "

	self nextPutAll: '<H' ,aNumber asString ,$>
		,aString ,'</H' ,aNumber asString ,$>! !

! PG8Demo methodsFor: 'io' !
title: aString
	" Emmit aString as tile on output view. "

	self title: aString level: 2! !

! PG8Demo methodsFor: 'callback' !
codeSending: aSymbol
	" Private - Return the code to send aSymbol to the receiver via callback. "

	^$',self class name,' current ', aSymbol ,$' ,'.doIt();'! !

! PG8Demo methodsFor: 'io' !
nextPutButton: title sending: aSymbol
	" Emmit button code to trigger aSymbol on the receiver when clicked. "

	self nextPutAll: '<button onclick="' ,(self codeSending: aSymbol) , '">' ,title,'</button>'! !

! PG8Demo methodsFor: 'io' !
reportError
	" Private - Return the error block reporting errors on outputView. "

	^[:error|
		self print: 'Error: ', error#message.
		self cr nextPutAll: 'Error: ',(PhoneGap descriptionof: error)
	]! !

! PG8Demo methodsFor: 'io' !
show: selectors of: anObject as: title clear: clear
	" Private - Show the values obtained sending selectors to anObject. "

	clear ifTrue: [ self clear ].
	title notNil ifTrue: [ self title: title ].
	selectors do: [:selector|
		(selector isKindOf: Array) ifTrue: [
			self	show: selector last of: (anObject perform: selector first)
				as: selector first clear: false
		] ifFalse: [
			self cr;nextPutAll: selector ,': ' ,(anObject perform: selector)
		]
	].! !

! PG8Demo methodsFor: 'io' !
show: selectors of: anObject as: title
	" Private - Show the values obtained sending selectors to anObject. "

	^self show: selectors of: anObject as: title clear: true! !

! PG8Demo methodsFor: 'io' !
list: selectors all: objects separator: separator useList: useList
	" Private - List information of objects. "

	useList ifTrue: [ self nextPutAll: '<ul>' ].
	objects do: [:object|
		useList ifTrue: [ self nextPutAll: '<li>' ] ifFalse: [ self cr ].
		selectors do: [:selector|
			self nextPutAll: selector ,'='
				,(object perform: selector) asString
				, separator
		].
		useList ifTrue: [ self nextPutAll: '</li>' ].
	].
	useList ifTrue: [ self nextPutAll: '</ul>' ].! !

! PG8Demo methodsFor: 'private' !
watch: anObject
	" Private - Set the watching instance.
	Release old instance if present.
	"

	watch release.
	watch := anObject.! !

! PG8Demo methodsFor: 'demo' !
showInformation
	" Show generic information provided by the library. "

	self show: #(
		#( #device #( #name #phonegap #platform #uuid #version ) )
		#( #connection #( #type ) )
	  ) of: PhoneGap as: #PhoneGap clear: true! !

! PG8Demo methodsFor: 'demo' !
show: anAsyncClass selectors: selectors as: title
	" Evaluate demostration of async query of data. "

	anAsyncClass
		do: [:aValue| self show: selectors of: aValue as: title ]
		onError: self reportError! !

! PG8Demo methodsFor: 'demo' !
watch: anAsyncClass selectors: selectors as: title
	" Start demostration of async watcher of data. "

	self watch: (anAsyncClass
		watch: [:aValue| self show: selectors of: aValue as: title ]
		onError: self reportError)! !

! PG8Demo methodsFor: 'demo' !
showAcceleration
	" Evaluate demostration action. "

	self show: Accelerometer selectors: #( #x #y #z #timeStamp ) as: 'Acceleration'! !

! PG8Demo methodsFor: 'demo' !
watchAcceleration
	" Evaluate demostration action. "

	self watch: Accelerometer selectors: #( #x #y #z #timeStamp ) as: 'Acceleration'! !

! PG8Demo methodsFor: 'demo' !
showCompass
	" Evaluate demostration action. "

	self show: Compass selectors: #( #magneticHeading #headingAccuracy #timeStamp ) as: 'CompassHeading'! !

! PG8Demo methodsFor: 'demo' !
watchCompass
	" Evaluate demostration action. "

	self watch: Compass selectors: #( #magneticHeading #headingAccuracy #timeStamp ) as: 'CompassHeading'! !

! PG8Demo methodsFor: 'demo' !
showGeolocation
	" Evaluate demostration action. "

	self show: Geolocation selectors: #( #( #coordinates #( #latitude #longitude #altitude #accuracy #altitudeAccuracy #heading #speed ) ) #timeStamp ) as: 'Geolocation'! !

! PG8Demo methodsFor: 'demo' !
watchGeolocation
	" Evaluate demostration action. "

	self watch: Geolocation selectors: #( #( #coordinates #( #latitude #longitude #altitude #accuracy #altitudeAccuracy #heading #speed ) ) #timeStamp ) as: 'Geolocation'! !

! PG8Demo methodsFor: 'demo' !
showCameraDemo
	" Show widgets for this demo. "

	self	clear; title: 'Camera Demo';
		nextPutButton: 'Capture photo' sending: #capturePhoto; "cr;"
		nextPutButton: 'Capture editable photo' sending: #captureEditablePhoto; cr;
		nextPutButton: 'From library...' sending: #capturePhotoLibrary; "cr;"
		nextPutButton: 'From album...' sending: #capturePhotoAlbum; hr;
		nextPutAll: '<img style="display:none;width:60px;height:60px;" id="demoSmallImage" src="" />'; cr;
		nextPutAll: '<img style="display:none" id="demoLargeImage" src="" />'; cr;
		yourself.! !

! PG8Demo methodsFor: 'demo' !
showCaptureDemo
	" Show widgets for this demo. "

	self	clear; title: 'Capture Demo';
		nextPutButton: 'Capture audio' sending: #captureAudio; "cr;"
		nextPutButton: 'Capture video' sending: #captureVideo; "cr;"
		nextPutButton: 'Capture image' sending: #captureImage; cr;
		nextPutAll: '<img style="display:none" id="demoCaptureImage" src="" />'; cr;
		yourself.! !

! PG8Demo methodsFor: 'demo' !
showImage: tagName src: data
	" Private - Update image element at tagName with data source. "

	| image |
	image := DOM document getElementById: tagName.
	image style display: 'block'.
	image src: data.! !

! PG8Demo methodsFor: 'demo' !
showMedia: type files: mediaFiles
	" Private - Report captured media files data. "

	self clear; title: 'Captured Media files of type ',type.
	mediaFiles do: [:each|
		self show: #( #name #fullPath #type #lastModifiedDate #size ) of: each as: '--' clear: false.
	].! !

! PG8Demo methodsFor: 'demo' !
showData: data
	" Private - Update data element. "

	self print: 'Data size=',data size.
	1024 * 4"kb" > data size ifTrue: [ self print: data ].! !

! PG8Demo methodsFor: 'private' !
imageDataHeader
	" Private - Return the image header string to send image data to DOMImage. "

	^'data:image/jpeg;base64,'! !

! PG8Demo methodsFor: 'demo' !
capturePhoto
	" Evaluate demostration action. "

	Camera
		do: [:data| self showdata: data. self showImage: #demoSmallImage src: self imageDataHeader ,data ]
		onError: self reportError with: #( #( #quality 50) )! !

! PG8Demo methodsFor: 'demo' !
captureEditablePhoto
	" Evaluate demostration action. "

	Camera
		do: [:data| self showdata: data. self showImage: #demoSmallImage src: self imageDataHeader ,data ]
		onError: self reportError with: #( #( #quality 20) #( #allowEdit true ) )! !

! PG8Demo methodsFor: 'demo' !
capturePhotoLibrary
	" Evaluate demostration action. "

	Camera
		do: [:uri| self showdata: uri. self showImage: #demoLargeImage src: uri ]
		onError: self reportError with: #( #( #quality 50) #( #destinationType 1 ) #( #sourceType 0 ) )! !

! PG8Demo methodsFor: 'demo' !
capturePhotoAlbum
	" Evaluate demostration action. "

	Camera
		do: [:uri| self showdata: uri. self showImage: #demoLargeImage src: uri ]
		onError: self reportError with: #( #( #quality 50) #( #destinationType 1 ) #( #sourceType 2 ) )! !

! PG8Demo methodsFor: 'demo' !
captureAudio
	" Evaluate demostration action. "

	CaptureAudio
		do: [:mediaFiles| self showMedia: #audio files: mediaFiles ]
		onError: self reportError with: #( #( #limit 2 ) )! !

! PG8Demo methodsFor: 'demo' !
captureVideo
	" Evaluate demostration action. "

	CaptureVideo
		do: [:mediaFiles| self showMedia: #video files: mediaFiles ]
		onError: self reportError with: #( #( #limit 2 ) )! !

! PG8Demo methodsFor: 'demo' !
captureImage
	" Evaluate demostration action. "

	CaptureImage
		do: [:mediaFiles| self showMedia: #image files: mediaFiles ]
		onError: self reportError with: #( #( #limit 2 ) )! !

! PG8Demo methodsFor: 'demo' !
reportDirectoryBlock: aDirectory
	" Private - Return the block to report entries of aDirectory. "

	^[:entries|
		self	cr; nextPutAll: aDirectory fullPath,' #entries=',entries size asString
				,'  ',(entries collect: [:entry| entry name ])asString;
			list: #( #name #isDirectory ) all: entries separator: ' ' useList: false;
			yourself.
		entries do: [:each|
			(self cancelled not and: [ each isDirectory ])
			ifTrue: [ each reader entriesDo: (self reportDirectoryBlock: each) onError: self reportError ]
		]
	]! !

! PG8Demo methodsFor: 'demo' !
reportRoot
	" Private - Return the block to report a fileSystem and root directory. "

	^[:fileSystem| | root |
		root := fileSystem root.
		self	title: fileSystem class name,' name=',fileSystem name;
			nextPutAll: 'Root[',root class name,'] name=',root name; cr;
			nextPutAll: 'Root fullPath=',root fullPath; cr;
			nextPutAll: 'Root uri=',root uri; cr;
			yourself.
		root reader entriesDo: (self reportDirectoryBlock: root) onError: self reportError.
	]! !

! PG8Demo methodsFor: 'demo' !
showFileSystemDemo
	" Show results of this demo. "

	self clear; nextPutButton: 'Cancel operation' sending: #cancel; cr.
	FileSystem
		temporaryDo: self reportRoot onError: self reportError;
		persistentDo: self reportRoot onError: self reportError;
		applicationDo: self reportRoot onError: self reportError;
		resourceDo: self reportRoot onError: self reportError;
		yourself.! !

! PG8Demo methodsFor: 'demo' !
alertDemo
	" Execute this demo. "

	PhoneGap notification
		alert: 'This is an alert message!!'
		doing: [ self cr;nextPutAll: 'Alert has been closed.' ]
		title: 'Alert title.'
		buttonName: 'Ok, I know.'! !

! PG8Demo methodsFor: 'demo' !
confirmDemo
	" Execute this demo. "

	PhoneGap notification
		confirm: 'Are you ok ?'
		doing: [:index| self clear; nextPutAll: 'Confirmation result = ',index asString ]
		title: 'Please confirm...' buttonNames: 'Yes,No,Cancel'! !

! PG8Demo methodsFor: 'demo' !
beepDemo
	" Execute this demo. "

	PhoneGap notification beep: 3! !

! PG8Demo methodsFor: 'demo' !
vibrateDemo
	" Execute this demo. "

	PhoneGap notification vibrate: 2000! !

! PG8Demo methodsFor: 'demo' !
showNotificationDemo
	" Show widgets for this demo. "

	self	clear; title: 'Notification Demo';
		nextPutButton: 'Alert' sending: #alertDemo; "cr;"
		nextPutButton: 'Confirm' sending: #confirmDemo; cr;
		nextPutButton: 'Beep' sending: #beepDemo; "cr;"
		nextPutButton: 'Vibrate' sending: #vibrateDemo; cr;
		yourself.! !

! PG8Demo methodsFor: 'demo' !
withDataBaseDo: aBlock
	" Private - Execute transaction block on demo database. "

	| database |
	database := PhoneGap database
		open: 'Database' version: '1.0'
		displayName: self class name size: 1000000.
	database do: aBlock onError: self reportError.! !

! PG8Demo methodsFor: 'demo' !
deleteDataBaseDemo
	" Execute this demo. "

	self withDataBaseDo: [:transaction|
		transaction , 'DROP TABLE IF EXISTS DEMO'
	]! !

! PG8Demo methodsFor: 'demo' !
createDataBaseDemo
	" Execute this demo. "

	self withDataBaseDo: [:transaction|
self print: 'Transaction=',transaction asString.
		transaction
			, 'DROP TABLE IF EXISTS DEMO'
			, 'CREATE TABLE IF NOT EXISTS DEMO (id unique, data)'
			, 'INSERT INTO DEMO (id, data) VALUES (1, "First row")'
			, 'INSERT INTO DEMO (id, data) VALUES (2, "Second row")'.
	]! !

! PG8Demo methodsFor: 'demo' !
showDataBaseRecord: result
	" Private - Show result of a query. "

	(JS @ result) propertyNames do: [:field|
		self nextPutAll: field ,'=' , (result basicAt: field);cr
	].! !

! PG8Demo methodsFor: 'demo' !
displayDataBaseDemo
	" Execute this demo. "

	self withDataBaseDo: [:transaction|
		transaction
			executeSql: 'SELECT * FROM DEMO'
			parameters: Array new
			onResult: [:result| self showDataBaseRecord: result ]
			onError: self reportError.
	]! !

! PG8Demo methodsFor: 'demo' !
showDataBaseDemo
	" Show widgets for this demo. "

	self	clear; title: 'DataBase Demo';
		nextPutButton: 'Create DEMO Table' sending: #createDataBaseDemo; "cr;"
		nextPutButton: 'Show DEMO Table' sending: #displayDataBaseDemo; "cr;"
		nextPutButton: 'Delete DEMO Table' sending: #deleteDataBaseDemo; cr;
		nextPutButton: 'Cancel operation' sending: #cancel; cr;
		yourself.! !

! PG8Demo methodsFor: 'demo' !
findFiles: maskArray in: aDirectory doing: aBlock
	" Private - Return the block to search entries of aDirectory for file matching maskArray. "

	| check |
	check := [:entry|
		maskArray do: [:mask|
			(entry name asLowercase includes: mask) ifTrue: [
				aBlock value: entry
			]
		]
	].
	^[:entries|
		entries do: [:each|
			self cancelled ifFalse: [
				each isDirectory
				ifTrue: [ each reader entriesDo: (self findFiles: maskArray in: each doing: aBlock) onError: self reportError ]
				ifFalse: [ check value: each ]
			].
		]
	]! !

! PG8Demo methodsFor: 'demo' !
findFilesFromRoot: maskArray doing: aBlock
	" Private - Return the block to search files in a fileSystem. "

	^[:fileSystem| | root |
		root := fileSystem root.
		root reader entriesDo: (self findFiles: maskArray in: root doing: aBlock) onError: self reportError.
	]! !

! PG8Demo methodsFor: 'demo' !
filesMatching: masks do: aBlock
	" Private - Evaluate aBlock for all files matching masks found in demo file system. "

	| maskArray |
	maskArray := (masks asArrayOfSubstringsSeparatedBy: $;) collect: [:each| each asLowercase ].
	maskArray notEmpty ifTrue: [
		FileSystem
			persistentDo: (self findFilesFromRoot: maskArray doing: aBlock)
			onError: self reportError
	]! !

! PG8Demo methodsFor: 'demo' !
showFileInfo: aPG8File
	" Private - Show information about aPG8File. "

	self show: #( #fullPath #type #size #lastModifiedDate ) of: aPG8File as: aPG8File name clear: false! !

! PG8Demo methodsFor: 'demo' !
showTextFile: aPG8File
	" Private - Show text contents of aPG8File. "

	| reader |
	self showFileInfo: aPG8File.
	reader := FileReader new.
	reader when: #loadEnd: do: [:event|
		self cr; nextPutAll: 'contents:<code>' ,reader result ,'</code>'
	].
	reader readAsText: aPG8File handle! !

! PG8Demo methodsFor: 'demo' !
showBinaryFile: aPG8File
	" Private - Show binary contents of aPG8File. "

	| reader |
	self showFileInfo: aPG8File.
	reader := FileReader new.
	reader when: #loadEnd: do: [:event|
		self cr; nextPutAll: 'contents:<code>' ,reader result ,'</code>'
	].
	reader readAsDataURL: aPG8File handle! !

! PG8Demo methodsFor: 'demo' !
readTextFileDemo
	" Execute this demo. "

	self clear.
	self filesMatching: '.txt' do: [:fileEntry|
		self cancel; cr; nextPutAll: #readTextFileDemo ,'  ',fileEntry fullPath.
		fileEntry fileDo: [:file| self showTextFile: file ] onError: self reportError.
	]! !

! PG8Demo methodsFor: 'demo' !
readJSONFileDemo
	" Execute this demo. "

	self clear.
	self filesMatching: '.json' do: [:fileEntry|
		self cancel; cr; nextPutAll: #readJSONFileDemo ,'  ',fileEntry fullPath.
		fileEntry fileDo: [:file| self showTextFile: file ] onError: self reportError.
	]! !

! PG8Demo methodsFor: 'demo' !
readBinaryFileDemo
	" Execute this demo. "

	self clear.
	self filesMatching: '.jpg;.png' do: [:fileEntry|
		self cancel; cr; nextPutAll: #readBinaryFileDemo ,'  ',fileEntry fullPath.
		fileEntry fileDo: [:file| self showBinaryFile: file ] onError: self reportError.
	]! !

! PG8Demo methodsFor: 'demo' !
writeTextFileDemo
	" Execute this demo. "

	self cr; nextPutAll: #writeTextFileDemo ,' not implemented yet' ! !

! PG8Demo methodsFor: 'demo' !
writeBinaryFileDemo
	" Execute this demo. "

	self cr; nextPutAll: #writeBinaryFileDemo ,' not implemented yet' ! !

! PG8Demo methodsFor: 'demo' !
showFileDemo
	" Show widgets for this demo. "

	self	clear; title: 'File read/write Demo';
		nextPutButton: 'Read text file' sending: #readTextFileDemo; "cr;"
		nextPutButton: 'Read JSON file' sending: #readJSONFileDemo; "cr;"
		nextPutButton: 'Read binary file' sending: #readBinaryFileDemo; cr;
		nextPutButton: 'Write text file' sending: #writeTextFileDemo; "cr;"
		nextPutButton: 'Write binary file' sending: #writeBinaryFileDemo; cr;
		nextPutButton: 'Cancel operation' sending: #cancel; cr;
		yourself.! !